unit uMain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Dialogs, Grids, ComCtrls;

type

  { TForm1 }

  TForm1 = class(TForm)
    ImageList1: TImageList;
    OpenDialog: TOpenDialog;
    SG: TStringGrid;
    StatusBar: TStatusBar;
    tbMenu: TToolBar;
    tbOpen: TToolButton;
    tbSave: TToolButton;
    tbSep1: TToolButton;
    tbInsertRowAbove: TToolButton;
    tbInsertRowBelow: TToolButton;
    tbDeleteRow: TToolButton;
    tbSep2: TToolButton;
    tbDeleteCol: TToolButton;
    tbInsertLeftCol: TToolButton;
    tbInsertRightCol: TToolButton;
    tbEdit: TToolButton;
    ToolButton1: TToolButton;
    procedure FormCreate(Sender: TObject);
    procedure SGClick(Sender: TObject);
    procedure SGColRowInserted(Sender: TObject; IsColumn: Boolean; sIndex,
      tIndex: Integer);
    procedure SGEditingDone(Sender: TObject);
    procedure SGKeyPress(Sender: TObject; var Key: char);
    procedure tbDeleteColClick(Sender: TObject);
    procedure tbDeleteRowClick(Sender: TObject);
    procedure tbEditClick(Sender: TObject);
    procedure tbInsertLeftColClick(Sender: TObject);
    procedure tbInsertRightColClick(Sender: TObject);
    procedure tbInsertRowAboveClick(Sender: TObject);
    procedure tbInsertRowBelowClick(Sender: TObject);
    procedure tbOpenClick(Sender: TObject);
    procedure tbSaveClick(Sender: TObject);
  private
    fTopRow: Integer;
    fIsEditable: Boolean;
    procedure LoadFileToGrid(AFilename: String);
    procedure SetColumnTitles;
    procedure GoToScrollPos(aTopRow, aRow: Integer);
    procedure ButtonStates(aState: Boolean);
  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.SGEditingDone(Sender: TObject);
begin
  SG.EditorMode := False;
end;

procedure TForm1.SGClick(Sender: TObject);
begin
  //StatusBar.Panels[2].Text := SG.TopRow.ToString;
  //StatusBar.Panels[3].Text := BoolToStr(fIsEditable, 'True', 'False');
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  fIsEditable := False;
  SG.EditorMode := False;
end;

procedure TForm1.SGColRowInserted(Sender: TObject; IsColumn: Boolean; sIndex,
  tIndex: Integer);
begin
  StatusBar.Panels[2].Text := 'sIndex: '+sIndex.ToString+', tIndex: '+tIndex.ToString;
  StatusBar.Panels[3].Text := BoolToStr(fIsEditable, 'True', 'False');
  if fIsEditable then
  begin
    //SG.SetFocus;
    //SG.EditorMode := True;
  end;
end;

procedure TForm1.SGKeyPress(Sender: TObject; var Key: char);
begin
  tbSave.Enabled := True;
end;

// Menubutton clicks
procedure TForm1.tbDeleteColClick(Sender: TObject);
begin
  if SG.Col <> -1 then
  begin
    SG.DeleteCol(SG.Col);
    SetColumnTitles;
  end;
end;

procedure TForm1.tbDeleteRowClick(Sender: TObject);
begin
  fTopRow := SG.TopRow;
  SG.DeleteRow(SG.Row); // delete row
  GoToScrollPos(fTopRow, SG.Row);
end;

procedure TForm1.tbEditClick(Sender: TObject);
begin
  if OpenDialog.FileName <> '' then
  begin
    ButtonStates(True);
  end;
end;

procedure TForm1.tbInsertLeftColClick(Sender: TObject);
begin
  SG.InsertColRow(True, SG.Col);
  SetColumnTitles;
end;

procedure TForm1.tbInsertRightColClick(Sender: TObject);
begin
  if (SG.Col + 1) = SG.ColCount then
    SG.ColCount := SG.ColCount + 1
  else
    SG.InsertColRow(True, SG.Col + 1);
  SetColumnTitles;
end;

procedure TForm1.tbInsertRowAboveClick(Sender: TObject);
begin
  SG.InsertColRow(False, SG.Row);
end;

procedure TForm1.tbInsertRowBelowClick(Sender: TObject);
begin
  SG.InsertColRow(False, SG.Row + 1);
end;

procedure TForm1.tbOpenClick(Sender: TObject);
begin
  OpenDialog.Execute;
  if OpenDialog.FileName <> '' then
  begin
    LoadFileToGrid(OpenDialog.FileName);
    tbEdit.Enabled := True;
  end;
end;

procedure TForm1.tbSaveClick(Sender: TObject);
var
  aRow, aCol: integer;
  tmp: string;
  sl: TStringList;
begin
  fTopRow := SG.TopRow;
  tmp := '';
  for aRow := 1 to SG.RowCount - 1 do
  begin
    for aCol := 0 to SG.ColCount - 1 do
    begin
       if SG.Cells[aCol, aRow] <> '' then
      begin
        // Großer Lugstein ;DA/SX-001 ;
        tmp += Trim(SG.Cells[aCol, aRow]) + ' ;';
      end;
    end;
    // remove the last semicolon
    // Großer Lugstein ;DA/SX-001
    tmp := Copy(tmp, 0, Length(tmp)-2);
    // add crlf
    tmp += #13#10;
  end;
  // save to file
  sl := TStringList.Create;
  try
    sl.Add(tmp);
    sl.SaveToFile(OpenDialog.FileName);
  finally
    sl.Free;
  end;
  // reload the stringgrid
  if OpenDialog.FileName <> '' then
    LoadFileToGrid(OpenDialog.FileName);

  GoToScrollPos(fTopRow, SG.Row);

  // Set button states
  ButtonStates(False);
  tbSave.Enabled := False;
end;


procedure TForm1.LoadFileToGrid(AFilename: String);
begin
  // init stringgrid
  SG.Clear;
  SG.EditorMode := false;
  SG.BeginUpdate;
  SG.LoadFromCSVFile(AFilename, ';', false, 0, True);
  SG.EndUpdate();

  SetColumnTitles;
  SG.AutoAdjustColumns;

  //
  StatusBar.Panels[0].Text := 'Cols: ' + IntToStr(SG.ColCount);
  StatusBar.Panels[1].Text := 'Rows: ' + IntToStr(SG.RowCount-1);
end;

procedure TForm1.SetColumnTitles;
var
  i: integer;
begin
  for i := 0 to SG.ColCount - 1 do
  begin
    SG.Cells[i,0] := 'Column' + IntToStr(i+1);
  end;
end;

procedure TForm1.GoToScrollPos(aTopRow, aRow: Integer);
begin
  SG.TopRow := aTopRow;
  SG.Row := aRow;
end;

procedure TForm1.ButtonStates(aState: Boolean);
begin
  SG.AutoEdit := aState;
  fIsEditable := aState;
  tbInsertRowAbove.Enabled := aState;
  tbInsertRowBelow.Enabled := aState;
  tbDeleteRow.Enabled := aState;
  tbInsertRightCol.Enabled := aState;
  tbInsertLeftCol.Enabled := aState;
  tbDeleteCol.Enabled := aState;
end;



end.

